# ---
# jupyter:
#   jupytext_format_version: '1.2'
#   kernelspec:
#     display_name: Python (jupyter virtualenv)
#     language: python
#     name: jupyter
#   language_info:
#     codemirror_mode:
#       name: ipython
#       version: 3
#     file_extension: .py
#     mimetype: text/x-python
#     name: python
#     nbconvert_exporter: python
#     pygments_lexer: ipython3
#     version: 3.6.5
# ---

# # Reading from Google Sheets
#
# There are complicated read/write possibilities (e.g. [this](https://socraticowl.com/post/integrate-google-sheets-and-jupyter-notebooks/) and [this](https://github.com/robin900/gspread-dataframe)), but on the assumption we are happy to share contents of sheets publicly, the easiest thing is to publish a worksheet in CSV format using [Google Sheets "publish" functionality](https://support.google.com/docs/answer/183965).
#

# +
import pandas as pd

# This URL obtained via the Google Sheets's `publish to the web` menu (File > Publish to the web;
# select the worksheet from the first dropdown and "comma-separated values" from the second;
# copy the link)
google_sheet = "https://docs.google.com/spreadsheets/d/e/2PACX-1vTefoHueV7W523NR2uc5ckS9zJFeuWFr2v84WkzThbobQq5KPKIqlued_UqRCy31YGhF4P3XEyJWlI8/pub?gid=252563648&single=true&output=csv"

pd.read_csv(google_sheet).head()



from ebmdatalab import charts
import pandas as pd
import numpy as np


def test_add_percentiles():
    df = pd.DataFrame(np.random.rand(1000, 1), columns=["val"])
    months = pd.date_range("2018-01-01", periods=12, freq="M")
    df["month"] = np.random.choice(months, len(df))
    df = charts.add_percentiles(df, period_column="month", column="val")
    # This is a statistically-likely test, so might fail!
    assert (df[df.percentile == 99].val > 0.75).all()


def test_deciles_chart():
    """Currently just tests it doesn't fail
    """
    df = pd.DataFrame(np.random.rand(1000, 1), columns=["val"])
    months = pd.date_range("2018-01-01", periods=12, freq="M")
    df["month"] = np.random.choice(months, len(df))

    charts.deciles_chart(
        df,
        period_column="month",
        column="val",
        title="thing",
        show_outer_percentiles=True,
    )



import glob
import os
import random
import re
from hashlib import md5
import string

import pandas as pd


def fingerprint_sql(sql):
    # remove comments
    sql = re.sub(r"--(.*)", "", sql)
    # remove whitespace
    sql = re.sub(r"\s", "", sql)
    # lowercase everything not in double quotes
    to_lowercase = sql[:]
    to_substitute = []
    for quoted in re.findall(r'".*?"', sql):
        to_lowercase = to_lowercase.replace(quoted, "{}")
        to_substitute.append(quoted)
    to_lowercase = to_lowercase.lower()
    sql = to_lowercase.format(*to_substitute)
    return md5(sql.encode("utf-8")).hexdigest()


def cached_read(sql, csv_path=None, use_cache=True, **kwargs):
    """Run SQL in BigQuery and return dataframe, caching results in a CSV
    file

    """
    assert csv_path, "You must supply csv_path"
    defaults = {"project_id": "ebmdatalab", "dialect": "standard"}
    defaults.update(kwargs)
    fingerprint = fingerprint_sql(sql)
    csv_dir, csv_filename = os.path.split(csv_path)
    fingerprint_path = os.path.join(
        csv_dir, "." + csv_filename + "." + fingerprint + ".tmp"
    )
    already_cached = os.path.exists(fingerprint_path)
    if use_cache and already_cached:
        df = pd.read_csv(csv_path)
    else:
        old_fingerprint_files = glob.glob(
            os.path.join(csv_dir, "." + csv_filename + ".*.tmp")
        )
        for f in old_fingerprint_files:
            os.remove(f)
        with open(fingerprint_path, "w") as f:
            f.write("File created by {}".format(__file__))
        temp_path = '{}.{}.tmp'.format(csv_path, _random_str(8))
        df = pd.read_gbq(sql, **defaults)
        df.to_csv(temp_path, index=False)
        os.replace(temp_path, csv_path)
    return df


def _random_str(length):
    return ''.join(
        [random.choice(string.ascii_lowercase) for _ in range(length)]
    )



from ebmdatalab import maps
import tempfile
import pandas as pd


def test_maps():
    """This test just tests nothing fails at the moment
    """
    df = pd.DataFrame([["99P", 0.3], ["13T", 1.2]], columns=["pct", "val"])
    plt = maps.ccg_map(df, title="foo", column="val")
    with tempfile.NamedTemporaryFile() as f:
        plt.savefig(f.name, format="png", dpi=300, bbox_inches="tight")



from ebmdatalab import stats
import pandas as pd
import random


def test_compute_regression():
    # build dataframe with predictable regression characteristics
    matrix = []
    multiple = 3
    for a in range(1, 50):
        # we want b + c to be a multiple of a
        target_val = a * multiple
        b = target_val / random.randint(1, target_val)
        c = target_val - b
        matrix.append([a, b, c])
    df = pd.DataFrame(matrix, columns=["A", "B", "C"])

    # test against it
    formula = "A ~ B + C"
    df2 = stats.compute_regression(df, formula=formula).round(5)

    col_b_data = df2.loc["B"]
    expected_ratio = round(1 / multiple, 5)
    assert col_b_data["p value"] == 0
    assert col_b_data["coefficient"] == expected_ratio
    assert col_b_data["conf_int_low"] == expected_ratio
    assert col_b_data["conf_int_high"] == expected_ratio
    assert col_b_data["p value"] == 0

    intercept_data = df2.loc["Intercept"]
    assert intercept_data["coefficient"] == 0



from mock import patch
from ebmdatalab import bq
from pandas import DataFrame
import tempfile


def test_fingerprint_sql():
    input_sql = 'select *, "Frob" from x -- comment\n' "where (a >= 4);"
    same_sql_different_caps = 'SELECT *, "Frob" from x -- comment\n' "where (a >= 4);"
    same_sql_different_quoted_caps = (
        'SELECT *, "frob" from x -- comment\n' "where (a >= 4);"
    )
    same_sql_different_comment = (
        'select *, "Frob" from x -- comment 2\n' "where (a >= 4);"
    )
    same_sql_different_whitespace = (
        'select *, "Frob" from x -- comment 2\n' "\n" " where (a >= 4);"
    )
    fingerprint = bq.fingerprint_sql(input_sql)

    assert fingerprint == bq.fingerprint_sql(same_sql_different_caps)
    assert fingerprint != bq.fingerprint_sql(same_sql_different_quoted_caps)
    assert fingerprint == bq.fingerprint_sql(same_sql_different_comment)
    assert fingerprint == bq.fingerprint_sql(same_sql_different_whitespace)


@patch("ebmdatalab.bq.pd.read_gbq")
def test_cached_read(mock_read_gbq):
    mock_read_gbq.return_value = DataFrame([{"a": 3}])
    sql = "select * from foobar"

    # Test identical SQL bypasses reading BQ
    with tempfile.NamedTemporaryFile() as csv_file:
        for _ in range(0, 2):
            df = bq.cached_read(sql, csv_path=csv_file.name)
        assert df.loc[0]["a"] == 3
        assert mock_read_gbq.call_count == 1

        # and now with `use_cache` param
        df = bq.cached_read(sql, csv_path=csv_file.name, use_cache=False)
        assert mock_read_gbq.call_count == 2


def _check_cached_read(csv_file, mock_read, sql, expected):
    mock_read.return_value = expected
    df = bq.cached_read(sql, csv_path=csv_file.name)
    assert str(df) == str(expected)


@patch("ebmdatalab.bq.pd.read_gbq")
def test_old_cache_markers_removed(mock_read_gbq):
    with tempfile.NamedTemporaryFile() as csv_file:
        # First, cause some sql to be cached
        inputs_and_outputs = [
            ("select * from foobar", DataFrame([{"a": 1}])),
            ("select * from foobar order by id", DataFrame([{"a": 2}])),
        ]
        _check_cached_read(csv_file, mock_read_gbq, *inputs_and_outputs[0])
        _check_cached_read(csv_file, mock_read_gbq, *inputs_and_outputs[1])
        _check_cached_read(csv_file, mock_read_gbq, *inputs_and_outputs[0])



import pandas as pd
import statsmodels.formula.api as smf


def compute_regression(df, formula=""):
    """Fit a regression model
    """
    # Uses R-style formula as described here
    # https://www.statsmodels.org/dev/example_formulas.html
    data = df.copy()

    lm = smf.ols(formula=formula, data=data).fit()

    # output regression coefficients and p-values:
    params = (
        pd.DataFrame(lm.params)
        .reset_index()
        .rename(columns={0: "coefficient", "index": "factor"})
    )
    pvals = (
        pd.DataFrame(lm.pvalues[[1, 2]])
        .reset_index()
        .rename(columns={0: "p value", "index": "factor"})
    )
    params = params.merge(pvals, how="left", on="factor").set_index("factor")

    # add confidence intervals
    conf = pd.DataFrame(data=lm.conf_int())
    conf.columns = ["conf_int_low", "conf_int_high"]
    return params.join(conf)



import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import matplotlib

# Legend locations for matplotlib
BEST = 0
UPPER_RIGHT = 1
UPPER_LEFT = 2
LOWER_LEFT = 3
LOWER_RIGHT = 4
RIGHT = 5
CENTER_LEFT = 6
CENTER_RIGHT = 7
LOWER_CENTER = 8
UPPER_CENTER = 9
CENTER = 10


def add_percentiles(df, period_column=None, column=None, show_outer_percentiles=True):
    """For each period in `period_column`, compute percentiles across that
    range.

    Adds `percentile` column.

    """
    deciles = np.arange(0.1, 1, 0.1)
    bottom_percentiles = np.arange(0.01, 0.1, 0.01)
    top_percentiles = np.arange(0.91, 1, 0.01)
    if show_outer_percentiles:
        quantiles = np.concatenate((deciles, bottom_percentiles, top_percentiles))
    else:
        quantiles = deciles
    df = df.groupby(period_column)[column].quantile(quantiles).reset_index()
    df = df.rename(index=str, columns={"level_1": "percentile"})
    # create integer range of percentiles
    df["percentile"] = df["percentile"].apply(lambda x: int(x * 100))
    return df


def deciles_chart(
    df,
    period_column=None,
    column=None,
    title="",
    ylabel="",
    show_outer_percentiles=True,
    show_legend=True,
    ax=None,
):
    """period_column must be dates / datetimes
    """
    sns.set_style("whitegrid", {"grid.color": ".9"})
    if not ax:
        fig, ax = plt.subplots(1, 1)
    df = add_percentiles(
        df,
        period_column=period_column,
        column=column,
        show_outer_percentiles=show_outer_percentiles,
    )
    linestyles = {
        "decile": {"color": "b", "line": "b--", "linewidth": 1, "label": "decile"},
        "median": {"color": "b", "line": "b-", "linewidth": 1.5, "label": "median"},
        "percentile": {
            "color": "b",
            "line": "b:",
            "linewidth": 0.8,
            "label": "1st-9th, 91st-99th percentile",
        },
    }
    label_seen = []
    for percentile in range(1, 100):  # plot each decile line
        data = df[df["percentile"] == percentile]
        add_label = False

        if percentile == 50:
            style = linestyles["median"]
            add_label = True
        elif show_outer_percentiles and (percentile < 10 or percentile > 90):
            style = linestyles["percentile"]
            if "percentile" not in label_seen:
                label_seen.append("percentile")
                add_label = True
        else:
            style = linestyles["decile"]
            if "decile" not in label_seen:
                label_seen.append("decile")
                add_label = True
        if add_label:
            label = style["label"]
        else:
            label = "_nolegend_"

        ax.plot(
            data[period_column],
            data[column],
            style["line"],
            linewidth=style["linewidth"],
            color=style["color"],
            label=label,
        )
    ax.set_ylabel(ylabel, size=15, alpha=0.6)
    if title:
        ax.set_title(title, size=18)
    # set ymax across all subplots as largest value across dataset
    ax.set_ylim([0, df[column].max() * 1.05])
    ax.tick_params(labelsize=12)
    ax.set_xlim(
        [df[period_column].min(), df[period_column].max()]
    )  # set x axis range as full date range

    plt.setp(ax.xaxis.get_majorticklabels(), rotation=90)
    ax.xaxis.set_major_formatter(matplotlib.dates.DateFormatter("%B %Y"))
    if show_legend:
        ax.legend(
            bbox_to_anchor=(1.1, 0.8),  # arbitrary location in axes
            #  specified as (x0, y0, w, h)
            loc=CENTER_LEFT,  # which part of the bounding box should
            #  be placed at bbox_to_anchor
            ncol=1,  # number of columns in the legend
            fontsize=12,
            borderaxespad=0.0,
        )  # padding between the axes and legend
        #  specified in font-size units
    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    plt.gcf().autofmt_xdate()
    return plt



# ---
# jupyter:
#   jupytext_format_version: '1.2'
#   kernelspec:
#     display_name: Python (jupyter virtualenv)
#     language: python
#     name: jupyter
#   language_info:
#     codemirror_mode:
#       name: ipython
#       version: 3
#     file_extension: .py
#     mimetype: text/x-python
#     name: python
#     nbconvert_exporter: python
#     pygments_lexer: ipython3
#     version: 3.6.5
# ---

# ## Logistic regression
#
#

# +
import numpy as np
import pandas as pd

from ebmdatalab import stats

# Three columns of random numbers
df = pd.DataFrame(np.random.rand(100, 3), columns=["A", "B", "C"])
formula = "A ~ B + C"
stats.compute_regression(df, formula=formula)



# ---
# jupyter:
#   jupytext_format_version: '1.2'
#   kernelspec:
#     display_name: Python (jupyter virtualenv)
#     language: python
#     name: jupyter
#   language_info:
#     codemirror_mode:
#       name: ipython
#       version: 3
#     file_extension: .py
#     mimetype: text/x-python
#     name: python
#     nbconvert_exporter: python
#     pygments_lexer: ipython3
#     version: 3.6.5
# ---

# ## Maps
#
# Draw a CCG map, optionally with London separated out.

# + {"scrolled": false}
import numpy as np
import pandas as pd
from ebmdatalab import maps
import matplotlib.pyplot as plt
from importlib import reload

reload(maps)

plt.figure(figsize=(12, 8))
df = pd.read_json("ccg_list_size.json")
df.columns = [
    "date",
    "pct",
    "ccg_name",
    "total_list_size",
]  # The CCG column must be named 'pct'
df = df[df["date"] == "2018-10-01"]
plt = maps.ccg_map(
    df,
    title="CCG list sizes",
    map_year="2019",  # leave this unset to select current year
    column="total_list_size",
    separate_london=True,
    london_layout="horizontal",
)
plt.show()
# -

# You can also show the map as a cartogram where the CCGs are sized according to their patient population

plt = maps.ccg_map(
    df,
    title="CCG list sizes",
    column="total_list_size",
    cartogram=True,
    separate_london=False,
)
plt.show()

# ## Deciles
#
# Given a dataframe with a date column and a values column, compute
# percentiles for each date and plot them in a line chart.

# +
from ebmdatalab import charts
import matplotlib.gridspec as gridspec

import importlib

importlib.reload(charts)

# make a datafrom with a date column and a values column
df = pd.DataFrame(np.random.rand(1000, 1), columns=["val"])
months = pd.date_range("2018-01-01", periods=12, freq="M")
df["month"] = pd.to_datetime(np.random.choice(months, len(df)))

charts.deciles_chart(
    df,
    period_column="month",
    column="val",
    title="Random values",
    ylabel="n",
    show_outer_percentiles=True,
    show_legend=True,
)

# Now add a single line against the deciles
df_subject = pd.DataFrame(np.random.rand(12, 1), columns=["val"]) * 100
df_subject["month"] = months
df_subject.set_index("month")

plt.plot(df_subject["month"], df_subject["val"], "r--")
plt.show()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['ebmdatalab', 'ebmdatalab.tests']

package_data = \
{'': ['*'], 'ebmdatalab': ['data/*']}

install_requires = \
['statsmodels',
 'seaborn',
 'pandas >= 0.24',
 'pandas-gbq >= 0.8.0',
 'geopandas',
 'descartes']

extras_require = \
{'test': ['pytest >=2.7.3']}

setup(name='ebmdatalab',
      version='0.0.20',
      description='Package for ebmdatalab jupyter notebook stuff',
      author='EBM DataLab',
      author_email='tech@ebmdatalab.net',
      url='https://github.com/ebmdatalab/datalab-pandas',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )



import glob
import geopandas as gpd
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from pathlib import Path


def ccg_map(
    df,
    title="",
    column=None,
    separate_london=False,
    london_layout="horizontal",
    cartogram=False,
    subplot_spec=None,
    show_legend=True,
    map_year=None,
    plot_options=None,
):
    """Draw a CCG map with London separated out
    """
    # Because this uses subplots to arrange London and England,
    # the only way to arrange nested subplots is via a subplotspec
    assert column, "You must specify a column name to plot"
    df = df.copy()
    # input df must have 'pct' column, plus others as specified
    data_dir = Path(__file__).parent / "data"

    # Add names and if it's London. Note the names in ccg_for_map must
    # match names in the CCG geojson, as that doesn't include codes at
    # the momemt
    names = pd.read_csv(data_dir / "ccg_for_map.csv")

    # Check we know about all the codes in the input data
    diff = np.setdiff1d(df["pct"], names["code"])
    if len(diff) > 0:
        raise BaseException(
            "Data contains CCG codes we don't know about: {}".format(diff)
        )

    df = df.merge(names[["code", "name", "is_london"]], left_on="pct", right_on="code")
    df = df.set_index("name")

    # Load map data
    cartogram_suffix = ""
    if cartogram:
        cartogram_suffix = "_cartogram"
    if map_year:
        map_file = data_dir / "ccgs{}_{}.json".format(cartogram_suffix, map_year)
    else:
        map_file = sorted(
            glob.glob(str(data_dir / "ccgs{}_2*.json".format(cartogram_suffix)))
        )[-1]
    ccgs = gpd.read_file(map_file)
    # Normalise names to match `ccg_fo_map` format (above)
    ccgs["name"] = ccgs["name"].str.upper()
    ccgs = ccgs.set_index("name")
    # Remove ones without geometry - these are (usually) federations
    # rather than individual CCGs
    ccgs = ccgs[~ccgs["geometry"].isnull()]

    # Check we can map all the CCGs named in the input data
    diff = np.setdiff1d(df.index, ccgs.index)
    if len(diff) > 0:
        raise BaseException("Data contains CCG names we can't map: {}".format(diff))

    # Join map with data
    gdf = ccgs.join(df, rsuffix="_orig")

    # Split into london and rest of England
    gdf_london = gdf[gdf["is_london"] == True]
    gdf_roe = gdf[gdf["is_london"] == False]

    # set common value limits for colour scale
    default_plot_options = {
        'vmin': df[column].min(),
        'vmax': df[column].max(),
        'edgecolor': "black",
        'linewidth': 0.1,
        'cmap': "OrRd",
    }

    if plot_options is None:
        plot_options = {}

    for k, v in default_plot_options.items():
        if k not in plot_options:
            plot_options[k] = v

    def plot(gdf, ax, title="", legend=True):
        gdf.plot(
            ax=ax,
            column=column,
            legend=legend,
            **plot_options
        )
        ax.set_aspect(1.63)
        if title:
            ax.set_title(title, size=14)
        ax.axis("off")

    fig = plt.gcf()
    if not subplot_spec:
        subplot_spec = gridspec.GridSpec(1, 1)[0]
    if separate_london:
        if london_layout == "horizontal":
            gs = gridspec.GridSpecFromSubplotSpec(
                nrows=1, ncols=2, width_ratios=[1, 2], subplot_spec=subplot_spec
            )
            ldn_ax = fig.add_subplot(gs[0, 0])
            roe_ax = fig.add_subplot(gs[0, 1])
        else:
            gs = gridspec.GridSpecFromSubplotSpec(
                nrows=2, ncols=1, height_ratios=[2, 1], subplot_spec=subplot_spec
            )
            roe_ax = fig.add_subplot(gs[0, 0])
            ldn_ax = fig.add_subplot(gs[1, 0])

        plot(
            gdf_roe,
            roe_ax,
            title="England (excluding London):\n{}".format(title),
            legend=show_legend,
        )
        plot(gdf_london, ldn_ax, title="London:\n{}".format(title), legend=False)
    else:
        ax = plt.subplot(subplot_spec)
        plot(gdf, ax, title=title, legend=show_legend)
    return plt



# ---
# jupyter:
#   jupytext_format_version: '1.2'
#   kernelspec:
#     display_name: Python (jupyter virtualenv)
#     language: python
#     name: jupyter
#   language_info:
#     codemirror_mode:
#       name: ipython
#       version: 3
#     file_extension: .py
#     mimetype: text/x-python
#     name: python
#     nbconvert_exporter: python
#     pygments_lexer: ipython3
#     version: 3.6.5
# ---

# # Laying things out in a grid
#
# Matplotlib doesn't make this easy!  There's also more than one way to do it, and some surprising exceptions. Things to bear in mind when doing layouts:
#
# * The terminology is as follows:
#   * **figure** - the entire graphic
#   * **subplot** (or, confusingly,  **axis**) - a chart within a figure, including legend, labels, etc
# * The default API is state-based. That is, any time you call a command, the previous commands matter, even if you're not explicitly referencing them. So `plt.figure()` makes a figure, `plt.add_subplot(111)` adds a subplot to that figure, etc
#
# [This is an excellent primer](http://jonathansoma.com/lede/algorithms-2017/classes/fuzziness-matplotlib/how-pandas-uses-matplotlib-plus-figures-axes-and-subplots/), worth reading all the way through.
#
# If you want to lay things out in grids that span colums, with varying widths, you need to use a `GridSpec`. You also need to do this if you want to nest subplots within each other.  Therefore, the recommended layout methods below always use gridspecs, because you *might as well*.

# +
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from ebmdatalab import maps, charts


# -

# ## Show two charts side-by-side

# +
# Make the figure. `figsize` is defined in inches; to vary how this
# is rendered to pixels, you can also provide a `dpi` argument
f = plt.figure(figsize=(12, 4))
f.suptitle("Sharing Y axis")

# This magically aligns dates
f.autofmt_xdate()

# Define a layout that is one row deep and two columns wide.
# Note that passing in a `figure` is optional due to the statefulness of the pyplot interface, mentioned above
layout = gridspec.GridSpec(1, 2, figure=f)

# Get references to the axes for each of the two cells, for us to
# pass to plotting functions later
left_plot = plt.subplot(layout[0])
right_plot = plt.subplot(layout[1], sharey=ax1)  # Share the Y axis
# ...and because it's shared, suppress ticks on the second chart
plt.setp(right_plot.get_yticklabels(), visible=False)


# make a datafrom with a date column and values columns
df = pd.DataFrame(np.random.rand(1000, 2), columns=["val1", "val2"])
months = pd.date_range("2018-01-01", periods=12, freq="M")
df["month"] = np.random.choice(months, len(df))

# Plot a deciles chart on the left axis
charts.deciles_chart(
    df,
    period_column="month",
    column="val1",
    title="Random values",
    ylabel="n",
    show_outer_percentiles=True,
    show_legend=False,
    ax=left_plot,
)

# ...and one on the right axis
charts.deciles_chart(
    df,
    period_column="month",
    column="val2",
    title="Random values 2",
    show_outer_percentiles=True,
    show_legend=True,
    ax=right_plot,
)
# -

# ## Showing a map and a chart side-by-side
#
# This is slightly weird, because our map function itself uses two subplots (one for the England map, and the other for the London breakout), and uou can only nest subplots if you do it within a `SubplotSpec`So, instead of passing a subplot through to the map-drawing function, we have to pass a subplot spec.
#
# It happens that `GridSpec` creates an array of `SubplotSpec`, so that's what we pass through.

# +
f = plt.figure(figsize=(16, 6))
f.suptitle("Map next to chart")
f.autofmt_xdate()

# Define a layout that is one row deep and two columns wide
layout = gridspec.GridSpec(1, 2, figure=f)

# Get subplot references for these two cells

# Per above, we have to pass a subplot_spec to maps
subplot_spec = layout[0]

# For the chart, an axis will suffice
ax2 = plt.subplot(layout[1])

# Put a map on the left
df = pd.read_json("ccg_list_size.json")
df.columns = [
    "date",
    "pct",
    "ccg_name",
    "total_list_size",
]  # The CCG column must be named 'pct'
maps.ccg_map(
    df,
    title="CCG list sizes",
    column="total_list_size",
    cartogram=True,
    separate_london=False,
    subplot_spec=subplot_spec,
)


# Put a chart on the right
df = pd.DataFrame(np.random.rand(1000, 2), columns=["val1", "val2"])
months = pd.date_range("2018-01-01", periods=12, freq="M")
df["month"] = np.random.choice(months, len(df))
charts.deciles_chart(
    df,
    period_column="month",
    column="val2",
    title="Random values 2",
    show_outer_percentiles=True,
    show_legend=True,
    ax=ax2,
)
plt.show()
